@@ -1,11 +1,10 @@
 package Monitoring::Livestatus::UNIX;
+use parent 'Monitoring::Livestatus';
 
-use 5.000000;
 use strict;
 use warnings;
-use IO::Socket::UNIX;
-use Carp;
-use base "Monitoring::Livestatus";
+use IO::Socket::UNIX ();
+use Carp qw/confess/;
 
 =head1 NAME
 
@@ -29,9 +28,9 @@ be a the C<socket> specification. Use either socker OR server.
 =cut
 
 sub new {
-    my $class = shift;
-    unshift(@_, "peer") if scalar @_ == 1;
-    my(%options) = @_;
+    my($class,@args) = @_;
+    unshift(@args, "peer") if scalar @args == 1;
+    my(%options) = @args;
     $options{'name'} = $options{'peer'} unless defined $options{'name'};
 
     $options{'backend'} = $class;
@@ -51,28 +50,45 @@ sub new {
 
 sub _open {
     my $self      = shift;
+
     if(!-S $self->{'peer'}) {
         my $msg = "failed to open socket $self->{'peer'}: $!";
         if($self->{'errors_are_fatal'}) {
-            croak($msg);
+            confess($msg);
         }
         $Monitoring::Livestatus::ErrorCode    = 500;
         $Monitoring::Livestatus::ErrorMessage = $msg;
         return;
     }
-    my $sock = IO::Socket::UNIX->new(
+    my $sock;
+    eval {
+        $sock = IO::Socket::UNIX->new(
                                         Peer     => $self->{'peer'},
-                                     );
-    if(!defined $sock or !$sock->connected()) {
-        my $msg = "failed to connect to $self->{'peer'} :$!";
-        if($self->{'errors_are_fatal'}) {
-            croak($msg);
+                                        Type     => IO::Socket::UNIX::SOCK_STREAM,
+                                        Timeout  => $self->{'connect_timeout'},
+                                    );
+        if(!defined $sock || !$sock->connected()) {
+            my $msg = "failed to connect to $self->{'peer'}: $!";
+            if($self->{'errors_are_fatal'}) {
+                confess($msg);
+            }
+            $Monitoring::Livestatus::ErrorCode    = 500;
+            $Monitoring::Livestatus::ErrorMessage = $msg;
+            return;
         }
+    };
+
+    if($@) {
         $Monitoring::Livestatus::ErrorCode    = 500;
-        $Monitoring::Livestatus::ErrorMessage = $msg;
+        $Monitoring::Livestatus::ErrorMessage = $@;
         return;
     }
 
+    if(defined $self->{'query_timeout'}) {
+        # set timeout
+        $sock->timeout($self->{'query_timeout'});
+    }
+
     return($sock);
 }
 
@@ -82,6 +98,7 @@ sub _open {
 sub _close {
     my $self = shift;
     my $sock = shift;
+    return unless defined $sock;
     return close($sock);
 }
 
@@ -90,11 +107,11 @@ sub _close {
 
 =head1 AUTHOR
 
-Sven Nierlein, E<lt>nierlein@cpan.orgE<gt>
+Sven Nierlein, 2009-present, <sven@nierlein.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009 by Sven Nierlein
+Copyright (C) by Sven Nierlein
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
